#include "log/CLog.h"
#include "std/stdio.h"
#include <stdarg.h>

enum class LogLevel
{
    DEBUG,
    INFO,
    WARN,
    ERROR
};

enum class Backend
{
    Serial,
    Console
};

namespace KernelLog
{
    void log(const char* function, LogLevel level, const char* message, ...) PRINTF_LIKE(3, 4);
    void logln(const char* function, LogLevel level, const char* message, ...) PRINTF_LIKE(3, 4);
}

void log_backend_serial(const char* function, LogLevel level, const char* message, va_list origin);
void log_backend_console(const char* function, LogLevel level, const char* message, va_list origin);
bool log_level_enabled(LogLevel level);
bool log_backend_enabled(Backend backend);

extern "C" void clog_log(const char* function, enum log_level level, const char* message, ...)
{
    if (!log_level_enabled((LogLevel)level)) return;
    va_list ap;
    va_start(ap, message);
    if (log_backend_enabled(Backend::Serial)) log_backend_serial(function, (LogLevel)level, message, ap);
    if (log_backend_enabled(Backend::Console)) log_backend_console(function, (LogLevel)level, message, ap);
    va_end(ap);
}

extern "C" void clog_logln(const char* function, enum log_level level, const char* message, ...)
{
    if (!log_level_enabled((LogLevel)level)) return;
    va_list ap;
    va_start(ap, message);
    if (log_backend_enabled(Backend::Serial))
    {
        log_backend_serial(function, (LogLevel)level, message, ap);
        printf("\n");
    }
    if (log_backend_enabled(Backend::Console))
    {
        log_backend_console(function, (LogLevel)level, message, ap);
        kprintf("\n");
    }
    va_end(ap);
}